@@ -11,14 +11,14 @@ use crate::cassandra_sys::cass_cluster_new;
 use crate::cassandra_sys::cass_cluster_set_connect_timeout;
 use crate::cassandra_sys::cass_cluster_set_connection_heartbeat_interval;
 use crate::cassandra_sys::cass_cluster_set_connection_idle_timeout;
-use crate::cassandra_sys::cass_cluster_set_contact_points;
+use crate::cassandra_sys::cass_cluster_set_contact_points_n;
 use crate::cassandra_sys::cass_cluster_set_core_connections_per_host;
-use crate::cassandra_sys::cass_cluster_set_credentials;
+use crate::cassandra_sys::cass_cluster_set_credentials_n;
 use crate::cassandra_sys::cass_cluster_set_latency_aware_routing;
 use crate::cassandra_sys::cass_cluster_set_latency_aware_routing_settings;
-use crate::cassandra_sys::cass_cluster_set_load_balance_dc_aware;
+use crate::cassandra_sys::cass_cluster_set_load_balance_dc_aware_n;
 use crate::cassandra_sys::cass_cluster_set_load_balance_round_robin;
-use crate::cassandra_sys::cass_cluster_set_local_address;
+use crate::cassandra_sys::cass_cluster_set_local_address_n;
 use crate::cassandra_sys::cass_cluster_set_max_concurrent_creation;
 use crate::cassandra_sys::cass_cluster_set_max_concurrent_requests_threshold;
 use crate::cassandra_sys::cass_cluster_set_max_connections_per_host;
@@ -38,6 +38,7 @@ use crate::cassandra_sys::cass_cluster_set_tcp_keepalive;
 use crate::cassandra_sys::cass_cluster_set_tcp_nodelay;
 use crate::cassandra_sys::cass_cluster_set_timestamp_gen;
 use crate::cassandra_sys::cass_cluster_set_token_aware_routing;
+use crate::cassandra_sys::cass_cluster_set_token_aware_routing_shuffle_replicas;
 use crate::cassandra_sys::cass_cluster_set_use_schema;
 use crate::cassandra_sys::cass_cluster_set_whitelist_filtering;
 use crate::cassandra_sys::cass_cluster_set_write_bytes_high_water_mark;
@@ -49,16 +50,17 @@ use crate::cassandra_sys::cass_session_new;
 use crate::cassandra_sys::cass_true;
 use crate::cassandra_sys::CassCluster as _Cluster;
 
+use std::ffi::CStr;
 use std::ffi::NulError;
-use std::ffi::{CStr, CString};
 use std::fmt;
 use std::fmt::Display;
 use std::iter::Map;
 use std::net::AddrParseError;
 use std::net::Ipv4Addr;
+use std::os::raw::c_char;
 use std::result;
 use std::str::FromStr;
-use time::Duration;
+use std::time::Duration;
 
 /// A CQL protocol version is just an integer.
 pub type CqlProtocol = i32;
@@ -120,8 +122,8 @@ impl Cluster {
     ///
     pub fn set_contact_points(&mut self, contact_points: &str) -> Result<&mut Self> {
         unsafe {
-            let s = CString::new(contact_points.clone())?;
-            let err = cass_cluster_set_contact_points(self.0, s.as_ptr());
+            let cp_ptr = contact_points.as_ptr() as *const c_char;
+            let err = cass_cluster_set_contact_points_n(self.0, cp_ptr, contact_points.len());
             err.to_result(self)
         }
     }
@@ -132,8 +134,8 @@ impl Cluster {
     /// Only numeric addresses are supported.
     pub fn set_local_address(&mut self, name: &str) -> Result<&mut Self> {
         unsafe {
-            let s = CString::new(name.clone())?;
-            let err = cass_cluster_set_local_address(self.0, s.as_ptr());
+            let name_ptr = name.as_ptr() as *const c_char;
+            let err = cass_cluster_set_local_address_n(self.0, name_ptr, name.len());
             err.to_result(self)
         }
     }
@@ -335,10 +337,9 @@ impl Cluster {
     ///
     ///
     /// Default: 5000ms
-    #[allow(cast_possible_truncation, cast_sign_loss)]
     pub fn set_connect_timeout(&mut self, timeout: Duration) -> &Self {
         unsafe {
-            cass_cluster_set_connect_timeout(self.0, timeout.num_milliseconds() as u32);
+            cass_cluster_set_connect_timeout(self.0, timeout.as_millis() as u32);
         }
         self
     }
@@ -347,10 +348,9 @@ impl Cluster {
     ///
     ///
     /// Default: 12000ms
-    #[allow(cast_possible_truncation, cast_sign_loss)]
     pub fn set_request_timeout(&mut self, timeout: Duration) -> &Self {
         unsafe {
-            cass_cluster_set_request_timeout(self.0, timeout.num_milliseconds() as u32);
+            cass_cluster_set_request_timeout(self.0, timeout.as_millis() as u32);
         }
         self
     }
@@ -358,9 +358,15 @@ impl Cluster {
     /// Sets credentials for plain text authentication.
     pub fn set_credentials(&mut self, username: &str, password: &str) -> Result<&mut Self> {
         unsafe {
-            let username_cstr = CString::new(username)?;
-            let password_cstr = CString::new(password)?;
-            cass_cluster_set_credentials(self.0, username_cstr.as_ptr(), password_cstr.as_ptr());
+            let username_ptr = username.as_ptr() as *const c_char;
+            let password_ptr = password.as_ptr() as *const c_char;
+            cass_cluster_set_credentials_n(
+                self.0,
+                username_ptr,
+                username.len(),
+                password_ptr,
+                password.len(),
+            );
         }
         Ok(self)
     }
@@ -394,10 +400,11 @@ impl Cluster {
     ) -> Result<&mut Self> {
         unsafe {
             {
-                let local_dc = CString::new(local_dc)?;
-                cass_cluster_set_load_balance_dc_aware(
+                let local_dc_ptr = local_dc.as_ptr() as *const c_char;
+                cass_cluster_set_load_balance_dc_aware_n(
                     self.0,
-                    local_dc.as_ptr(),
+                    local_dc_ptr,
+                    local_dc.len(),
                     used_hosts_per_remote_dc,
                     if allow_remote_dcs_for_local_cl {
                         cass_true
@@ -433,6 +440,26 @@ impl Cluster {
         self
     }
 
+    /// Configures token-aware routing to randomly shuffle replicas.
+    /// This can reduce the effectiveness of server-side caching, but it
+    /// can better distribute load over replicas for a given partition key.
+    ///
+    ///
+    /// Default: true (enabled)
+    ///
+    ///
+    /// Note: Token-aware routing must be enabled for the setting to be
+    /// applicable.
+    pub fn set_token_aware_routing_shuffle_replicas(&mut self, enabled: bool) -> &Self {
+        unsafe {
+            cass_cluster_set_token_aware_routing_shuffle_replicas(
+                self.0,
+                if enabled { cass_true } else { cass_false },
+            );
+        }
+        self
+    }
+
     /// Configures the cluster to use latency-aware request routing or not.
     ///
     ///
@@ -464,7 +491,6 @@ impl Cluster {
     ///  <li>update_rate_ms: 100 milliseconds</li>
     ///  <li>min_measured: 50</li>
     /// </ul>
-    #[allow(cast_sign_loss)]
     pub fn set_latency_aware_routing_settings(
         &mut self,
         exclusion_threshold: f64,
@@ -477,9 +503,9 @@ impl Cluster {
             cass_cluster_set_latency_aware_routing_settings(
                 self.0,
                 exclusion_threshold,
-                scale.num_milliseconds() as u64,
-                retry_period.num_milliseconds() as u64,
-                update_rate.num_milliseconds() as u64,
+                scale.as_millis() as u64,
+                retry_period.as_millis() as u64,
+                update_rate.as_millis() as u64,
                 min_measured,
             );
         }
@@ -519,13 +545,12 @@ impl Cluster {
     ///
     ///
     /// Default: false (disabled).
-    #[allow(cast_possible_truncation, cast_sign_loss)]
     pub fn set_tcp_keepalive(&mut self, enable: bool, delay: Duration) -> &Self {
         unsafe {
             cass_cluster_set_tcp_keepalive(
                 self.0,
                 if enable { cass_true } else { cass_false },
-                delay.num_seconds() as u32,
+                delay.as_secs() as u32,
             );
         }
         self
@@ -550,10 +575,9 @@ impl Cluster {
     ///
     ///
     /// Default: 30 seconds
-    #[allow(cast_possible_truncation, cast_sign_loss)]
     pub fn set_connection_heartbeat_interval(&mut self, hearbeat: Duration) -> &mut Self {
         unsafe {
-            cass_cluster_set_connection_heartbeat_interval(self.0, hearbeat.num_seconds() as u32);
+            cass_cluster_set_connection_heartbeat_interval(self.0, hearbeat.as_secs() as u32);
             self
         }
     }
@@ -563,10 +587,9 @@ impl Cluster {
     ///
     ///
     /// Default: 60 seconds
-    #[allow(cast_possible_truncation, cast_sign_loss)]
     pub fn set_connection_idle_timeout(&mut self, timeout: Duration) -> &mut Self {
         unsafe {
-            cass_cluster_set_connection_idle_timeout(self.0, timeout.num_seconds() as u32);
+            cass_cluster_set_connection_idle_timeout(self.0, timeout.as_secs() as u32);
             self
         }
     }
