@@ -132,35 +132,55 @@ impl CassResult {
         unsafe { cass_result_has_more_pages(self.0) == cass_true }
     }
 
-    /// Sets the statement's paging state. This can be used to get the next page of
-    /// data in a multi-page query.
+    /// Gets the statement's paging state. This can be used to get the next page of
+    /// data in a multi-page query, by using `set_paging_state_token`.
     ///
-    /// <b>Warning:</b> The paging state should not be exposed to or come from
-    /// untrusted environments. The paging state could be spoofed and potentially
-    // used to gain access to other data.
-    pub fn set_paging_state_token(&mut self, paging_state: &str) -> Result<&mut Self> {
+    /// Returns:
+    ///   - `Ok(None)` if there are no more pages, and thus no paging state token.
+    ///   - `Ok(Some(Vec<u8>)) if there are more pages, and a paging state token.
+    ///   - `Err(_)` if there was an error getting the paging state token.
+    ///
+    /// [`set_paging_state_token`]: Statement::set_paging_state_token
+    pub fn paging_state_token(&self) -> Result<Option<Vec<u8>>> {
+        if !self.has_more_pages() {
+            return Ok(None);
+        }
+
         unsafe {
-            let state = CString::new(paging_state)?;
+            let mut token_ptr = mem::zeroed();
+            let mut token_length = mem::zeroed();
             cass_result_paging_state_token(
                 self.0,
-                &mut state.as_ptr(),
-                &mut (state.to_bytes().len()),
+                &mut token_ptr,
+                &mut token_length,
             )
-            .to_result(self)
+            .to_result(())
+            .map(|_| {
+                    Some(
+                        slice::from_raw_parts(token_ptr as *const u8, token_length as usize)
+                            .to_vec(),
+                    )
+                })
         }
     }
 
     /// Creates a new iterator for the specified result. This can be
     /// used to iterate over rows in the result.
     pub fn iter(&self) -> ResultIterator {
-        unsafe { ResultIterator(cass_iterator_from_result(self.0), PhantomData) }
+        unsafe {
+            ResultIterator(
+                cass_iterator_from_result(self.0),
+                cass_result_row_count(self.0),
+                PhantomData,
+            )
+        }
     }
 }
 
 /// An iterator over the results of a query.
 /// The result holds the data, so it must last for at least the lifetime of the iterator.
 #[derive(Debug)]
-pub struct ResultIterator<'a>(pub *mut _CassIterator, PhantomData<&'a CassResult>);
+pub struct ResultIterator<'a>(pub *mut _CassIterator, usize, PhantomData<&'a CassResult>);
 
 // The underlying C type has no thread-local state, but does not support access
 // from multiple threads: https://datastax.github.io/cpp-driver/topics/#thread-safety
@@ -173,7 +193,7 @@ impl<'a> Drop for ResultIterator<'a> {
 }
 
 impl<'a> Iterator for ResultIterator<'a> {
-    type Item = Row;
+    type Item = Row<'a>;
     fn next(&mut self) -> Option<<Self as Iterator>::Item> {
         unsafe {
             match cass_iterator_next(self.0) {
@@ -182,17 +202,21 @@ impl<'a> Iterator for ResultIterator<'a> {
             }
         }
     }
+
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        (0, Some(self.1))
+    }
 }
 
 impl<'a> ResultIterator<'a> {
     /// Gets the next row in the result set
-    pub fn get_row(&mut self) -> Row {
+    pub fn get_row(&mut self) -> Row<'a> {
         unsafe { Row::build(cass_iterator_get_row(self.0)) }
     }
 }
 
 impl<'a> IntoIterator for &'a CassResult {
-    type Item = Row;
+    type Item = Row<'a>;
     type IntoIter = ResultIterator<'a>;
 
     fn into_iter(self) -> Self::IntoIter {
